from typing import Any

from pysubway.model.role import Role
from pysubway.model.role_permission import RolePermission
from pysubway.model.user import User


class PermissionFactory:

    def __new__(self, style: str) -> Any:
        if style.lower() == 'db':
            return PermissionDB
        raise NotImplementedError(f'the style {style} is not supported now.')


class PermissionBase:
    """
    username, email and phone must be all unique.
    """

    def can_do(self, block_uuid: str, username: str = '', email: str = '', phone: str = '') -> None:
        raise NotImplementedError('')


class PermissionDB:

    def __init__(self, User: User, Role: Role, RolePermission: RolePermission):
        self.User = User
        self.Role = Role
        self.RolePermission = RolePermission

    def can_do(self, block_uuid: str, username: str = '', email: str = '', phone: str = '') -> bool:
        role_uuid = self.User.get_role(username=username, email=email, phone=phone)
        if not role_uuid:
            return False
        return self.RolePermission.the_role_can_do(role_uuid, block_uuid)



from collections import OrderedDict
from typing import Dict, Optional, Tuple, Any

import pandas as pd
from pysubway.errors import RequestFailed
from pysubway.utils import udict
from pysubway.utils.file import Dir
from pysubway.utils.file import File
from pysubway.utils.uexcel import Excel

from .base import BoolDataBase


class LiabilitiesDetails(BoolDataBase):
    interface_chinese_name = '负债详情'
    service = 'multiple_platforms_loan_v2_0'
    mode = 'mode_multiple_platforms_liabilities_detail'
    default_part = {'name': '',
                    'phone': '',
                    'ident_number': ''
                    }
    df_default_header_dict = OrderedDict({'name': '姓名',
                                          'phone': '手机号',
                                          'ident_number': '身份证号',
                                          'cflenders': '消费金融类机构数',
                                          'd7_lend_number': '近7天负债机构数',
                                          'liabilities_mechanism_number': '负债机构总数',
                                          'loanday': '首次负债距今时长',
                                          'm1_lend_number': '近1个月负债机构数',
                                          'm3_lend_number': '近3个月负债机构数',
                                          'm6_lend_number': '近6个月负债机构数',
                                          'nllenders': '网络贷款类机构数',
                                          'settlement_number': '已结清负债机构数'
                                          })

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'LiabilitiesDetails':
        self.biz_data = self.generate_biz_data(name, str(phone), str(ident_number))
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}),
                                 self.default_part,
                                 append_biz_data=append_biz_data)


class RepaymentDetail(BoolDataBase):
    interface_chinese_name = '还款详情'
    service = 'multiple_platforms_loan_v2_0'
    mode = 'mode_multiple_platforms_repayment_detail'
    default_part = {'name': '',
                    'phone': '',
                    'ident_number': ''
                    }
    df_default_header_dict = OrderedDict({'name': '姓名',
                                          'phone': "手机",
                                          'ident_number': '身份证',
                                          'd7_repay_fail_cnt': '近7天还款失败次数',
                                          'd7_repay_fail_money': '近7天还款失败金额',
                                          'd7_repay_succ_cnt': '近7天成功还款次数',
                                          'd7_repay_succ_money': '近7天成功还款金额',
                                          'last_repay_day': '最近一次成功还款距今天数',
                                          'm1_repay_fail_cnt': '近1个月还款失败次数',
                                          'm1_repay_fail_money': '近1个月还款失败金额',
                                          'm1_repay_succ_cnt': '近1个月成功还款次数',
                                          'm1_repay_succ_money': '近1个月成功还款金额',
                                          'm3_repay_fail_cnt': '近3个月还款失败次数',
                                          'm3_repay_fail_money': '近3个月还款失败金额',
                                          'm3_repay_succ_cnt': '近3个月成功还款次数',
                                          'm3_repay_succ_money': '近3个月成功还款金额',
                                          'm6_repay_fail_cnt': '近6个月还款失败次数',
                                          'm6_repay_fail_money': '近6个月还款失败金额',
                                          'm6_repay_succ_cnt': '近6个月成功还款次数',
                                          'm6_repay_succ_money': '近6个月成功还款金额'
                                          })

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'RepaymentDetail':
        # phone not str, parameter error
        self.biz_data = self.generate_biz_data(name, str(phone), str(ident_number))
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}),
                                 self.default_part,
                                 append_biz_data=append_biz_data)


class OverdueDetail(BoolDataBase):
    interface_chinese_name = '逾期详情'
    service = 'multiple_platforms_loan_v2_0'
    mode = 'mode_multiple_platforms_overdue_detail'
    default_part = {'name': '',
                    'phone': '',
                    'ident_number': ''
                    }
    df_default_header_dict = OrderedDict({'name': '姓名',
                                          'phone': '手机',
                                          'ident_number': '身份证',
                                          'counts': '逾期机构数',
                                          'is_overdue': '是否逾期',
                                          'overdue_money': '逾期金额',
                                          'settlement': '是否存在逾期未结清'
                                          })

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'OverdueDetail':
        self.biz_data = self.generate_biz_data(name, str(phone), str(ident_number))
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}),
                                 self.default_part,
                                 append_biz_data=append_biz_data)


class ApplyDetail(BoolDataBase):
    interface_chinese_name = '申请详情'
    service = 'multiple_platforms_loan_v2_0'
    mode = 'mode_multiple_platforms_apply_detail'
    default_part = {'name': '',
                    'phone': '',
                    'ident_number': ''
                    }
    df_default_header_dict = OrderedDict({'name': '姓名',
                                          'phone': '手机',
                                          'ident_number': '身份证',
                                          'apply_mechanism_number': '历史查询调用平台数量',
                                          'd7_apply_times': '近7天查询调用次数',
                                          'm12_apply_times': '近12个月查询调用次数',
                                          'm1_apply_times': '近1个月查询调用次数',
                                          'm3_apply_times': '近3个月查询调用次数',
                                          'm6_apply_times': '近6个月查询调用次数'
                                          })

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'ApplyDetail':
        self.biz_data = self.generate_biz_data(name, str(phone), str(ident_number))
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}),
                                 self.default_part,
                                 append_biz_data=append_biz_data)


def batch_test_apply_detail(input: str,
                            input_is_str: bool,
                            account: Dict[str, str],
                            output_file: str,
                            names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
                            columns: Optional[OrderedDict] = None,
                            raise_failed_exception: bool = False,
                            error_log: Optional[str] = '',
                            ) -> None:
    """

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    column = columns or ApplyDetail.df_default_header_dict
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    if input_is_str:
        print('batch_test_apply_detail excel_ins', '*' * 10, input, names, '*' * 10)
        excel_ins: pd.DataFrame = Excel.prepare_file(input).read(names=names)
        print('batch_test_apply_detail excel_ins', '*' * 10, excel_ins.dataframe.to_string(), '*' * 10)
    else:
        excel_ins = Excel(input).read(names=names).dataframe if isinstance(input, str) else input
    result_df = pd.DataFrame()
    guard_ins = ApplyDetail(account['rsa_prv_key'], account['aes_key'], account['company_uuid'])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        guard_result = guard_ins.set_biz_data(name, phone, idcard).request()
        if guard_ins.is_incoming_data_error():
            raise ImportError(f'response {guard_ins.response} series {series.values}')
        if not guard_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        guard_result_df: pd.DataFrame = guard_ins.result_as_df_partial()
        guard_result_df.rename(columns=column, inplace=True)
        result_df = result_df.append(guard_result_df)
        # break
    print('batch_test_apply_detail', '->', result_df.to_string(), output_file, column, '<-')
    result_df.to_excel(output_file, columns=column.values(), index=False)


def batch_test_overdue_detail(input: str,
                              input_is_str: bool,
                              account: Dict[str, str],
                              output_file: str,
                              names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
                              columns: Optional[OrderedDict] = None,
                              raise_failed_exception: bool = False,
                              error_log: Optional[str] = None,
                              ) -> None:
    """

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    columns = columns or OverdueDetail.df_default_header_dict
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    if input_is_str:
        print('batch_test_apply_detail excel_ins', '*' * 10, input, names, '*' * 10)
        excel_ins: pd.DataFrame = Excel.prepare_file(input).read(names=names, skiprows=0)
        print('batch_test_apply_detail excel_ins', '*' * 10, excel_ins.dataframe.to_string(), '*' * 10)
    else:
        excel_ins = Excel(input).read(names=names).dataframe if isinstance(input, str) else input
    result_df = pd.DataFrame()
    guard_ins = OverdueDetail(account['rsa_prv_key'], account['aes_key'], account['company_uuid'])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        guard_result = guard_ins.set_biz_data(name, phone, idcard).request()
        if guard_ins.is_incoming_data_error():
            raise ImportError(f'response {guard_ins.response} series {series.values}')
        if not guard_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        guard_result_df: pd.DataFrame = guard_ins.result_as_df_partial()
        guard_result_df.rename(columns=columns, inplace=True)
        result_df = result_df.append(guard_result_df)
        # break
    result_df.to_excel(output_file, columns=columns.values(), index=False)


def batch_test_repayment_detail(input: str,
                                input_is_str: bool,
                                account: Dict[str, str],
                                output_file: str,
                                names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
                                columns: Optional[OrderedDict] = None,
                                raise_failed_exception: bool = False,
                                error_log: Optional[str] = None,
                                ) -> None:
    """

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    columns = columns or RepaymentDetail.df_default_header_dict
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    if input_is_str:
        print('batch_test_apply_detail excel_ins', '*' * 10, input, names, '*' * 10)
        excel_ins: pd.DataFrame = Excel.prepare_file(input).read(names=names, skiprows=0)
        print('batch_test_apply_detail excel_ins', '*' * 10, excel_ins.dataframe.to_string(), '*' * 10)
    else:
        excel_ins = Excel(input).read(names=names).dataframe if isinstance(input, str) else input
    result_df = pd.DataFrame()
    guard_ins = RepaymentDetail(account['rsa_prv_key'], account['aes_key'], account['company_uuid'])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        guard_result = guard_ins.set_biz_data(name, phone, idcard).request()
        if not guard_ins.is_succeed():
            if guard_ins.is_incoming_data_error():
                raise ImportError(f'response {guard_ins.response} series {series.values}')
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        guard_result_df: pd.DataFrame = guard_ins.result_as_df_partial()
        guard_result_df.rename(columns=columns, inplace=True)
        result_df = result_df.append(guard_result_df)
        # break
    result_df.to_excel(output_file, columns=columns.values(), index=False)


def batch_test_liabilities_detail(input: str,
                                  input_is_str: bool,
                                  account: Dict[str, str],
                                  output_file: str,
                                  names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
                                  columns: Optional[OrderedDict] = None,
                                  raise_failed_exception: bool = False,
                                  error_log: Optional[str] = None,
                                  ) -> None:
    """

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    columns = columns or LiabilitiesDetails.df_default_header_dict
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    # it's support to raw string and file.
    if input_is_str:
        print('batch_test_apply_detail excel_ins', '*' * 10, input, names, '*' * 10)
        excel_ins: pd.DataFrame = Excel.prepare_file(input).read(names=names, skiprows=0)
        print('batch_test_apply_detail excel_ins', '*' * 10, excel_ins.dataframe.to_string(), '*' * 10)
    else:
        excel_ins = Excel(input).read(names=names).dataframe if isinstance(input, str) else input
    result_df = pd.DataFrame()
    guard_ins = LiabilitiesDetails(account['rsa_prv_key'], account['aes_key'], account['company_uuid'])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        guard_result = guard_ins.set_biz_data(name, phone, idcard).request()
        if not guard_ins.is_succeed():
            if guard_ins.is_incoming_data_error():
                raise ImportError(f'response {guard_ins.response} series {series.values}')
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        guard_result_df: pd.DataFrame = guard_ins.result_as_df_partial()
        guard_result_df.rename(columns=columns, inplace=True)
        result_df = result_df.append(guard_result_df)
        # break
    result_df.to_excel(output_file, columns=columns.values(), index=False)


def batch_test_used_web(input: str,
                        input_is_str: bool,
                        account: Dict[str, str],
                        output_dir: str,
                        names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
                        raise_failed_exception: bool = True,
                        error_log: Optional[str] = None,
                        ) -> None:
    dir = Dir(output_dir)
    dir.mkdir()
    if not dir.is_dir():
        raise ValueError(f'output_dir {output_dir} must be dir')
    output_liabilities_detail = File.join_path(output_dir, LiabilitiesDetails.interface_chinese_name + '.xlsx')
    batch_test_liabilities_detail(input,
                                  input_is_str,
                                  account,
                                  output_liabilities_detail,
                                  names=names,
                                  raise_failed_exception=raise_failed_exception,
                                  error_log=error_log)
    output_repayment_detail = File.join_path(output_dir, RepaymentDetail.interface_chinese_name + '.xlsx')
    batch_test_repayment_detail(input,
                                input_is_str,
                                account,
                                output_repayment_detail,
                                names=names,
                                raise_failed_exception=raise_failed_exception,
                                error_log=error_log)
    output_dir_overdue_detail = File.join_path(output_dir, OverdueDetail.interface_chinese_name + '.xlsx')
    batch_test_overdue_detail(input,
                              input_is_str,
                              account,
                              output_dir_overdue_detail,
                              names=names,
                              raise_failed_exception=raise_failed_exception,
                              error_log=error_log)
    output_dir_apply_detail = File.join_path(output_dir, ApplyDetail.interface_chinese_name + '.xlsx')
    batch_test_apply_detail(input,
                            input_is_str,
                            account,
                            output_dir_apply_detail,
                            names=names,
                            raise_failed_exception=raise_failed_exception,
                            error_log=error_log)
    dir.zip()



# # example
# import json
# import traceback
# from typing import Any, Optional, Dict
#
# import flask
# from flask import abort
# from flask.views import MethodView
# from flask_cors import CORS
# from flask_sqlalchemy import SQLAlchemy
# from pysubway.cache import FlaskCache
# from pysubway.errors import IncomingDataError, RequestFailed
# from pysubway.model.guard.company import Company
# from pysubway.model.guard.company_product import CompanyProduct
# from pysubway.model.guard.product import Product
# from pysubway.service.booldata.base import BoolDataBase
# from pysubway.service.booldata.hz_risk_model import HzRiskModel
# from pysubway.utils.file import File
# from pysubway.utils.file import FileIni
# from pysubway.view import View
# from pysubway.view import ViewUsedCodeBookAndUrlToken
#
# BOOLDATA_ACCOUNT = dict(
#     rsa_prv_key="""-----BEGIN RSA PRIVATE KEY-----
# MIIEowIBAAKCAQEA3PgggZllwepRByXqAeqrWU4Zx7m+0kvhcbEvpbGTez7PAj8u
# IJRKGnH2QfOkZHsRknamuBtJr9c2Js9yrrWbazNPFQrjekprroBSC7DO+EUHQqxZ
# mV8ldqukT+C/UOHXavNH3FHnrYSgU7yj4I1+AnAzQq33xbSASE9DkTyoEzJnlRjn
# TBopidDVA+P/bxR9z6FBWv8M6P/VAcKamYGIwsx9AbsVhHDd8yOzMDJHOuJza4ku
# KFyjf/N+2Fwc7d6ZbKhRYITio7nerw6/XYrcWhAAyB6XhEp3wfCA5SpgCBWGbdve
# fiPsme5qqirCy2rlPn76Biev3wW2zhVWTw7HDwIDAQABAoIBABA3vdsFMSy8T9og
# dD5TxOO3EblQ7qpsm01G3eJhWBuxjmvxyybk+1NZjeNlSGl/htijELVue0gGmZjb
# nOUpuxBxIZq/w9ZT4/dYv6zP+0DJgDDqiWDyVMOS8WpTanc7PB5DYMDQ2hooI8RB
# kh2HBPqU1Y5NSmQeTVQBTUo5k3RlGbeAE05U3kVfg6j6xf/67rceCOKa6G+Tkp6h
# eebLmuuc19rqg7jBblCsoMgb76P+STlnpvyB8fIYkb1W64bHkX/m4Qj/f4SiDh5t
# 9W3/bmB+2ygsQCaTPGS9y6z8EfnWZT74Fd+o8XuStubEFw0q+b/BdrmEfL5CjEnO
# q57Ep4ECgYEA4/rO82UlKQ6YS0pv4Z7yqPSrJSYwtoazGbGtgp1EHuRk//VEfLOT
# 4Yo665CbBIrTJxfgZlNkSkBoYaZA8KBg1fP56iuCBUvqBRhSvy94E8dNYjLEjX1q
# gskOFrfJ2UaHdp6alctrNSIuWd5q/1LIA4qUBD+mLbnRVWC6dIV2W68CgYEA+CC9
# Zw3M1w+5EYWXgq7D6PiDXMjGgvVn+J0/9nDSxVi41zhnxGQD40h/9eNAai67rJHG
# xvjK313aUVvMJRSXZuR9Y3zwh1Qb6wUdFgDlGUMuJbazVEgNQDrexf9oCGkvlmPH
# /5WDHTOdch07rNaE6/0xnSk45intCIbZy3bEQqECgYACHlWH+3uh6wnNQU7S2OhG
# W6eve7BeMdg+N+F14kI8y0CJBF1zjzOjl+Y+RCS8oRGfPmCOct3utrSBm8rksYjU
# 1CSRYYAeznrJO1Whgy5peKOmcvRSoES6HGYuHd0ZUMd3ebfUBoTjhILLwP5biwhi
# yAniFDlWGXDxiymQZaEY7wKBgQC1r6OAdkZro4OsJNoQQtAJIw9tPUa+VdvP27Ue
# SvbKsnCGhzLURzfhkRDBh6l/FsEtV4Dp6PsEG8Dkmgcdj7pdCSoXU0D8lsXqjVAC
# 8JzQprR/wQwGxsuNII5HBQG0RvEleeLUbzIWi9jZ2jX8FQg3lKpjne+f+UsrVzLb
# HRgQgQKBgDeMsUFxAeNFN9iHu0IqUboe09ULrZ0NZMHOJ3zoFnuus4sEQpNDC32y
# TuzAQumL7z6U5EzzKfdOZX1aAaLoId6+WNjaYkaSD+r/Iz8EzA8V3maip4M2usDl
# NLm+JyWqvj9Ze9y7VOgIqRarqjgNwjBoWysNzKIOMpxxItEWUE6b
# -----END RSA PRIVATE KEY-----""",
#     aes_key='vo8N6ZPawqLDhGj7',
#     company_uuid='01b20490-4a1e-11e9-9eb7-247703d210dc',
# )
#
# DEV_DB = dict(
#     host='47.102.132.18',
#     user='root',
#     password='qwe123456',
#     database='lightning',
# )
#
# PROD_DB = dict(
#     # open
#     host='rm-uf6xfyemz1085v2a48o.mysql.rds.aliyuncs.com',
#     # inner
#     # host='rm-uf6xfyemz1085v2a4.mysql.rds.aliyuncs.com',
#     user='root',
#     password='Qwe123456',
#     database='lighting',
#     inner_platform='inner_platform',
# )
#
# EMAIL_LIST = [
#     'booleandatamsg@shouxin168.com',
#     'booleandatamsg002@shouxin168.com',
#     'booleandatamsg003@shouxin168.com',
#     'booleandatamsg004@shouxin168.com',
#     'booleandatamsg005@shouxin168.com',
# ]
#
# EMAIL_PASSWORD = 'Qwe123456_'
#
# # remove mypy hints: error: Value of type "object" is not indexable
# DB_CONF: Dict[str, Any] = {
#     'SQLALCHEMY_DATABASE_URI':
#         f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["inner_platform"]}?charset=utf8',
#     'SQLALCHEMY_BINDS': {
#         'guard': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8',
#         'default': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}'
#         f'/{PROD_DB["inner_platform"]}?charset=utf8',
#     },
#     'SQLALCHEMY_ECHO': True,
# }
#
# print(DB_CONF)
#
# app = flask.Flask(__name__)
# CACHE = FlaskCache(app)
# app.config.from_mapping(DB_CONF)
# SQLAlchemy().init_app(app)
# SQLAlchemy().create_all(app=app)
# HASH_SALT = 'sdaer3rf'
# KEY_USERNAME = 'userName'
# KEY_PASSWORD = 'password'
# CODE_BOOK = f'{File.this_dir(__file__)}/static/code_book.ini'
# THIS_DIR = File.this_dir(__file__)
# KEY_ACCESS_TOKEN = 'access_token'
# VIEW_INS = ViewUsedCodeBookAndUrlToken(CODE_BOOK,
#                                        CACHE,
#                                        KEY_USERNAME,
#                                        KEY_PASSWORD,
#                                        key_access_token=KEY_ACCESS_TOKEN,
#                                        hash_salt=HASH_SALT)
# # inner platform web module
# MODULE_ACCOUNT_USERNAME = 'account_username'
# MODULE_ACCOUNTS_AUTHENTICATION = 'accounts_authentication'
# # ++++++++++++++++++++++++
# MODULE_EMAIL_NOTIFICATION = 'email_notification'
# OPTION_BATCH_TEST = 'batch_test'
# # ++++++++++++++++++++++++
# MODULE_RECHARGE_MONEY_ACCOUNT = 'recharge_money_account'
# # ++++++++++++++++++++++++
# MODULE_OPEN_TEST_ACCOUNT = 'open_company_test_account'
# # +++++++++++++++++++++++
# MODULE_INNER_MODULES = 'inner_modules'
# MODULE_USER_MANAGER = 'user_manager'
# MODULE_PRODUCT_BATCH_TEST = 'product_batch_test'
#
# DING_TIMOR = 'https://oapi.dingtalk.com/robot/send?access_token=ad34f7787f6232a5c82ccdd30210565ae0de828bc8566d0aebf1e5becf03d263'
# DING_CHI_JIAO_DA_XIAN = 'https://oapi.dingtalk.com/robot/send?access_token=73caa869cc06fc5bb37bf853d7d782784f09dd3705d0c31ade9852f4cf444c25'
# DEFAULT_RECEIVERS = VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default')
# REDIS_DEV = 'redis://47.102.132.18:6379/10'
# from celery import Celery
#
# celery = Celery(main='async_task')
# celery.config_from_object('async_task.conf')
# celery.add_defaults({
#     'BROKER_URL': REDIS_DEV,
#     'CELERY_RESULT_BACKEND': REDIS_DEV,
# })
# # ASYNC_TASK_ENGINE = generate_celery(REDIS_DEV)
# ASYNC_TASK_BATCH_TEST_HUIZUMODEL = 'batch_test_huizumodel'
# from async_task.task_notification import send_dingding_text, echo_hello
#
#
# def get_real_name(account: str) -> str:
#     return VIEW_INS.get_code_book_val(MODULE_ACCOUNT_USERNAME, account)
#
#
# def open_guard_test_account() -> str:
#     # todo open_guard_test_account
#     return ''
#
#
# @app.before_request
# def verify_token() -> Optional[flask.Response]:
#     """
#     :key cache key is access_token
#     :return:
#     """
#     return VIEW_INS.verify_before_request()
#
#
# # todo
#
# class BatchTest(MethodView):
#
#     @staticmethod
#     def generate_send_msg(real_name, remarks):
#         title = '布尔数据个人信用查询测试申请'
#         content = f'申请者: {real_name}\n测试理由：{remarks}' if remarks else ''
#         return title, content
#
#     @classmethod
#     def send_email_batch_test(cls,
#                               account: str = '',
#                               remarks: str = '',
#                               filename: str = '',
#                               receivers: str = '') -> None:
#         real_name = get_real_name(account)
#         title, content = cls.generate_send_msg(real_name, remarks)
#         celery.send_task('send_email_attach',
#                          (EMAIL_LIST[0], EMAIL_PASSWORD, title, content, filename, receivers))
#
#     @staticmethod
#     def get_company_uuid() -> str:
#         return VIEW_INS.login_cache_password
#
#     def len_biz_data(self, biz_data: str) -> int:
#         return len(biz_data.split('\n')) - 1
#
#     def batch_tes(self, biz_data, company_uuid):
#         result, output_file = self.batch_test_huizumodel(BOOLDATA_ACCOUNT,
#                                                          env='prod',
#                                                          company_uuid=company_uuid,
#                                                          biz_data=biz_data)
#         return result
#
#     def batch_test_platformsv2(self):
#         pass
#
#     def product_table(self, product):
#         table = {
#             'huizumodel': ASYNC_TASK_BATCH_TEST_HUIZUMODEL,
#             'multi_platformsv2': self.batch_test_platformsv2
#         }
#         batch_test = table.get(product)
#         if not batch_test:
#             raise NotImplementedError(f'product {product} not implements')
#         return batch_test
#
#     def post(self) -> str:
#         VIEW_INS.check_operation_authority(MODULE_PRODUCT_BATCH_TEST)
#         account = VIEW_INS.login_cache_username
#         try:
#             company_uuid = self.get_company_uuid()
#             if not CompanyProduct.has_permission_for_product(company_uuid,
#                                                              HzRiskModel.service,
#                                                              HzRiskModel.mode):
#                 raise IncomingDataError(f'company_uuid {company_uuid} do not has_permission_for_product')
#             biz_data, product, remarks, specified_receivers = View.request.get_data('biz_data',
#                                                                                     'product',
#                                                                                     'remarks',
#
#                                                                                     'email')
#             print('xxxxxxxxxxxxx++++++++++++++++++')
#             echo_hello.delay()
#             print('*' * 10, biz_data, product, remarks, specified_receivers)
#             # async_task_name = self.product_table(product)
#             real_name = get_real_name(account)
#             title, content = self.generate_send_msg(real_name, remarks)
#             echo_hello.delay()
#             # result, output_file = '', ''
#             # s = celery.send_task('send_dingding_text',
#             #                                 args=(
#             #                                     DING_TIMOR,
#             #                                     f'{title}\n{content}\n测试了{self.len_biz_data(biz_data)}条.'),
#             #                                 )
#             # print('s' * 10, s)
#             # send_dingding_text.delay(DING_TIMOR, 'xxxxxxxxxx')
#             # w = ASYNC_TASK_ENGINE.send_task(name=ASYNC_TASK_BATCH_TEST_HUIZUMODEL,
#             #                                 args=(BOOLDATA_ACCOUNT,
#             #                                       'prod',
#             #                                       company_uuid,
#             #                                       biz_data),
#             #                                 link=[
#             #                                     signature(
#             #                                         'send_email_attach',
#             #                                         (EMAIL_LIST[0], EMAIL_PASSWORD, title, content, output_file,
#             #                                          specified_receivers)),
#             #                                     signature(
#             #                                         'send_email_attach',
#             #                                         (EMAIL_LIST[0], EMAIL_PASSWORD, title, content, output_file,
#             #                                          DEFAULT_RECEIVERS)),
#             #                                 ],
#             #                                 link_error=signature('send_dingding_text', (DING_CHI_JIAO_DA_XIAN, '错误'))
#             #                                 )
#             # print('-' * 10, w)
#             # s = ASYNC_TASK_ENGINE.send_task('send_dingding_text',
#             #                                 args=(
#             #                                     DING_CHI_JIAO_DA_XIAN,
#             #                                     f'{title}\n{content}\n测试了{self.len_biz_data(biz_data)}条.'),
#             #                                 )
#             # print('s' * 10, s)
#             return View.response.success()
#         except (IncomingDataError, NotImplementedError) as e:
#             traceback.print_exc()
#             return abort(403)
#         except RequestFailed as e:
#             traceback.print_exc()
#
#             def parse_error(e: Exception) -> object:
#                 return json.loads(str(e))
#
#             error = parse_error(e)
#             if isinstance(error, dict) and error.get('resp_code') in BoolDataBase.resp_code_biz_data_error:
#                 return abort(403)
#             else:
#                 celery.send_task('send_dingding_text',
#                                             (DING_TIMOR, traceback.format_exc())
#                                             )
#                 return abort(500)
#         except Exception as e:
#             traceback.print_exc()
#             celery.send_task('send_dingding_text',
#                                         (DING_TIMOR, traceback.format_exc())
#                                         )
#             return abort(500)
#
#
# app.add_url_rule('/batch-test-huizu/', view_func=BatchTest.as_view('batch_test_huizu'))
#
#
# @app.route('/login', methods=['POST'])
# def login() -> flask.Response:
#     return VIEW_INS.login_with_cache()
#
#
# @app.route('/companies', methods=['get'])
# def display_companies() -> flask.Response:
#     return View.response.success({ins.uuid: ins.name for ins in Company.get_all()})
#
#
# @app.route('/products', methods=['get'])
# def get_all_products() -> flask.Response:
#     return View.response.success({'/'.join((ins.service_name, ins.mode)): ins.name for ins in Product.get_all()})
#
#
# @app.route('/products/v2', methods=['get'])
# def get_all_products_v2() -> flask.Response:
#     result = []
#     for ins in Product.get_all():
#         result.append({
#             'index': '/'.join((ins.service_name, ins.mode)),
#             'name': ins.name,
#             'price': '0',
#         })
#     return View.response.success(result)
#
#
# @app.route('/modules', methods=['get'])
# def get_all_modules() -> flask.Response:
#     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
#     print('get_all_modules >>>>>', FileIni(CODE_BOOK).items(MODULE_INNER_MODULES))
#     return VIEW_INS.response.success(dict(FileIni(CODE_BOOK).items(MODULE_INNER_MODULES)))
#
#
# if __name__ == '__main__':
#     CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)
#     app.run(port=8889, host='0.0.0.0', debug=False, use_reloader=True, processes=True)



# example

import json
import traceback
from typing import Any, Dict, Union, Tuple

import flask
from celery import Celery
from flask import abort
from flask.views import MethodView
from flask_cors import CORS
from flask_sqlalchemy import SQLAlchemy
from pysubway.async_task.task_notification import emailAbout, dingUrl
from pysubway.cache import FlaskCache
from pysubway.errors import IncomingDataError, RequestFailed
from pysubway.model.guard.company import Company
from pysubway.model.guard.company_product import CompanyProduct
from pysubway.model.guard.product import Product
from pysubway.service.booldata.base import BoolDataBase
from pysubway.service.booldata.hz_risk_model import HzRiskModel
from pysubway.utils.file import File
from pysubway.utils.file import FileIni
from pysubway.utils.ustring import unique_id
from pysubway.view import View
from pysubway.view import ViewUsedCodeBookAndUrlToken

# ********************************* config start ***************************
BOOLDATA_ACCOUNT = dict(
    rsa_prv_key="""-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEA3PgggZllwepRByXqAeqrWU4Zx7m+0kvhcbEvpbGTez7PAj8u
IJRKGnH2QfOkZHsRknamuBtJr9c2Js9yrrWbazNPFQrjekprroBSC7DO+EUHQqxZ
mV8ldqukT+C/UOHXavNH3FHnrYSgU7yj4I1+AnAzQq33xbSASE9DkTyoEzJnlRjn
TBopidDVA+P/bxR9z6FBWv8M6P/VAcKamYGIwsx9AbsVhHDd8yOzMDJHOuJza4ku
KFyjf/N+2Fwc7d6ZbKhRYITio7nerw6/XYrcWhAAyB6XhEp3wfCA5SpgCBWGbdve
fiPsme5qqirCy2rlPn76Biev3wW2zhVWTw7HDwIDAQABAoIBABA3vdsFMSy8T9og
dD5TxOO3EblQ7qpsm01G3eJhWBuxjmvxyybk+1NZjeNlSGl/htijELVue0gGmZjb
nOUpuxBxIZq/w9ZT4/dYv6zP+0DJgDDqiWDyVMOS8WpTanc7PB5DYMDQ2hooI8RB
kh2HBPqU1Y5NSmQeTVQBTUo5k3RlGbeAE05U3kVfg6j6xf/67rceCOKa6G+Tkp6h
eebLmuuc19rqg7jBblCsoMgb76P+STlnpvyB8fIYkb1W64bHkX/m4Qj/f4SiDh5t
9W3/bmB+2ygsQCaTPGS9y6z8EfnWZT74Fd+o8XuStubEFw0q+b/BdrmEfL5CjEnO
q57Ep4ECgYEA4/rO82UlKQ6YS0pv4Z7yqPSrJSYwtoazGbGtgp1EHuRk//VEfLOT
4Yo665CbBIrTJxfgZlNkSkBoYaZA8KBg1fP56iuCBUvqBRhSvy94E8dNYjLEjX1q
gskOFrfJ2UaHdp6alctrNSIuWd5q/1LIA4qUBD+mLbnRVWC6dIV2W68CgYEA+CC9
Zw3M1w+5EYWXgq7D6PiDXMjGgvVn+J0/9nDSxVi41zhnxGQD40h/9eNAai67rJHG
xvjK313aUVvMJRSXZuR9Y3zwh1Qb6wUdFgDlGUMuJbazVEgNQDrexf9oCGkvlmPH
/5WDHTOdch07rNaE6/0xnSk45intCIbZy3bEQqECgYACHlWH+3uh6wnNQU7S2OhG
W6eve7BeMdg+N+F14kI8y0CJBF1zjzOjl+Y+RCS8oRGfPmCOct3utrSBm8rksYjU
1CSRYYAeznrJO1Whgy5peKOmcvRSoES6HGYuHd0ZUMd3ebfUBoTjhILLwP5biwhi
yAniFDlWGXDxiymQZaEY7wKBgQC1r6OAdkZro4OsJNoQQtAJIw9tPUa+VdvP27Ue
SvbKsnCGhzLURzfhkRDBh6l/FsEtV4Dp6PsEG8Dkmgcdj7pdCSoXU0D8lsXqjVAC
8JzQprR/wQwGxsuNII5HBQG0RvEleeLUbzIWi9jZ2jX8FQg3lKpjne+f+UsrVzLb
HRgQgQKBgDeMsUFxAeNFN9iHu0IqUboe09ULrZ0NZMHOJ3zoFnuus4sEQpNDC32y
TuzAQumL7z6U5EzzKfdOZX1aAaLoId6+WNjaYkaSD+r/Iz8EzA8V3maip4M2usDl
NLm+JyWqvj9Ze9y7VOgIqRarqjgNwjBoWysNzKIOMpxxItEWUE6b
-----END RSA PRIVATE KEY-----""",
    aes_key='vo8N6ZPawqLDhGj7',
    company_uuid='01b20490-4a1e-11e9-9eb7-247703d210dc',
)

DEV_DB = dict(
    host='47.102.132.18',
    user='root',
    password='qwe123456',
    database='lightning',
)

PROD_DB = dict(
    # open
    host='rm-uf6xfyemz1085v2a48o.mysql.rds.aliyuncs.com',
    # inner
    # host='rm-uf6xfyemz1085v2a4.mysql.rds.aliyuncs.com',
    user='root',
    password='Qwe123456',
    database='lighting',
    inner_platform='inner_platform',
)

EMAIL_LIST = [
    'booleandatamsg@shouxin168.com',
    'booleandatamsg002@shouxin168.com',
    'booleandatamsg003@shouxin168.com',
    'booleandatamsg004@shouxin168.com',
    'booleandatamsg005@shouxin168.com',
]

EMAIL_PASSWORD = 'Qwe123456_'

# remove mypy hints: error: Value of type "object" is not indexable
DB_CONF: Dict[str, Any] = {
    'SQLALCHEMY_DATABASE_URI':
        f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["inner_platform"]}?charset=utf8',
    'SQLALCHEMY_BINDS': {
        'guard': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8',
        'default': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}'
        f'/{PROD_DB["inner_platform"]}?charset=utf8',
    },
    'SQLALCHEMY_ECHO': True,
}
# ********************************* config end ***************************

# ********************************* constant variable start ***************************
app = flask.Flask(__name__)
CACHE = FlaskCache(app)
app.config.from_mapping(DB_CONF)
SQLAlchemy().init_app(app)
# SQLAlchemy().create_all(app=app)
HASH_SALT = 'sdaer3rf'
KEY_USERNAME = 'userName'
KEY_PASSWORD = 'password'
CODE_BOOK = f'{File.this_dir(__file__)}/static/code_book.ini'
THIS_DIR = File.this_dir(__file__)
KEY_ACCESS_TOKEN = 'access_token'
VIEW_INS = ViewUsedCodeBookAndUrlToken(CODE_BOOK,
                                       CACHE,
                                       KEY_USERNAME,
                                       KEY_PASSWORD,
                                       key_access_token=KEY_ACCESS_TOKEN,
                                       hash_salt=HASH_SALT)
# inner platform web module
MODULE_ACCOUNT_USERNAME = 'account_username'
MODULE_ACCOUNTS_AUTHENTICATION = 'accounts_authentication'
# ++++++++++++++++++++++++
MODULE_EMAIL_NOTIFICATION = 'email_notification'
OPTION_BATCH_TEST = 'batch_test'
# ++++++++++++++++++++++++
MODULE_RECHARGE_MONEY_ACCOUNT = 'recharge_money_account'
# ++++++++++++++++++++++++
MODULE_OPEN_TEST_ACCOUNT = 'open_company_test_account'
# +++++++++++++++++++++++
MODULE_INNER_MODULES = 'inner_modules'
MODULE_USER_MANAGER = 'user_manager'
MODULE_PRODUCT_BATCH_TEST = 'product_batch_test'

DING_TIMOR = 'https://oapi.dingtalk.com/robot/send?access_token=ad34f7787f6232a5c82ccdd30210565ae0de828bc8566d0aebf1e5becf03d263'
DING_CHI_JIAO_DA_XIAN = 'https://oapi.dingtalk.com/robot/send?access_token=73caa869cc06fc5bb37bf853d7d782784f09dd3705d0c31ade9852f4cf444c25'
DING_QI_BAO = 'https://oapi.dingtalk.com/robot/send?access_token=a37dd3149f79bc903ed5b2135dfe988f2e32bf652c85de04b4e531b2669d9ea9'
DEFAULT_RECEIVERS = VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default')
REDIS_DEV = 'redis://47.102.132.18:6379/10'
RESULT_BACKEND = 'db+mysql+pymysql://root:Qwe123456@rm-uf6xfyemz1085v2a48o.mysql.rds.aliyuncs.com/inner_platform?charset=utf8'
APP_CELERY = Celery('async_task')
APP_CELERY.config_from_object('async_task.conf')
APP_CELERY.add_defaults({
    'BROKER_URL': REDIS_DEV,
    'CELERY_RESULT_BACKEND': REDIS_DEV,
})


# ********************************* constant variable end ***************************

# ********************************* tool function start ***************************
def get_real_name(account: str) -> str:
    val = VIEW_INS.get_code_book_val(MODULE_ACCOUNT_USERNAME, account)
    if not isinstance(val, str):
        raise TypeError(f'account {account} get val {val} is not str')
    return val


def open_guard_test_account() -> str:
    # todo open_guard_test_account
    return ''


# ********************************* tool function end ***************************

# ********************************* view function start ***************************
@app.before_request
def verify_token() -> Union[flask.Response, str, None]:
    """
    :key cache key is access_token
    :return:
    """
    return VIEW_INS.verify_before_request()


class BatchTest(MethodView):

    @staticmethod
    def generate_send_msg(real_name: str, remarks: str) -> Tuple[str, str]:
        title = '布尔数据个人信用查询测试申请'
        content = f'申请者: {real_name}\n测试理由：{remarks}' if remarks else ''
        return title, content

    @staticmethod
    def get_company_uuid() -> str:
        return VIEW_INS.login_cache_password

    def len_biz_data(self, biz_data: str) -> int:
        return len(biz_data.split('\n')) - 1

    @staticmethod
    def product_table(key: str) -> str:
        products = {
            'huizumodel': 'batch_test_huizumodel',
            'multi_platforms_v2': 'multi_platforms_v2',

        }
        try:
            return products[key]
        except KeyError:
            raise NotImplementedError(f'product {key} not exist')

    def post(self) -> flask.Response:
        VIEW_INS.check_operation_authority(MODULE_PRODUCT_BATCH_TEST)
        account = VIEW_INS.login_cache_username
        try:
            company_uuid = self.get_company_uuid()
            if not CompanyProduct.has_permission_for_product(company_uuid,
                                                             HzRiskModel.service,
                                                             HzRiskModel.mode):
                raise IncomingDataError(f'company_uuid {company_uuid} do not has_permission_for_product')
            biz_data, product, remarks, specified_receivers = View.request.get_data('biz_data',
                                                                                    'product',
                                                                                    'remarks',
                                                                                    'email')
            real_name = get_real_name(account)
            title, content = self.generate_send_msg(real_name, remarks)
            print('-' * 10)
            email_about = emailAbout(mail_user=EMAIL_LIST[0],
                                     mail_pass=EMAIL_PASSWORD,
                                     title=title,
                                     content=content,
                                     filename='',
                                     receivers=DEFAULT_RECEIVERS,
                                     specified_receivers=specified_receivers)
            # todo for test
            ding_url = dingUrl(ding_msg=DING_TIMOR, ding_exception=DING_TIMOR)
            # ding_url = dingUrl(ding_msg=DING_CHI_JIAO_DA_XIAN, ding_exception=DING_TIMOR)
            async_task_name = self.product_table(product)
            if async_task_name == 'batch_test_huizumodel':
                APP_CELERY.send_task(name=async_task_name,
                                     args=(BOOLDATA_ACCOUNT,
                                           'prod',
                                           company_uuid,
                                           biz_data,
                                           File.this_dir(__file__)),
                                     kwargs={'email_about': email_about,
                                             'ding_url': ding_url})
            elif async_task_name == "multi_platforms_v2":
                print(f'it is calling {async_task_name}, biz_data {biz_data}')
                APP_CELERY.send_task(name=async_task_name,
                                     kwargs={
                                         'input': biz_data,
                                         'input_is_str': True,
                                         'account': BOOLDATA_ACCOUNT,
                                         'names': ('name', 'phone', 'idcard'),
                                         'raise_failed_exception': True,
                                         'output_dir': File.join_path(File.this_dir(__file__), unique_id()),
                                         'email_about': email_about,
                                         'ding_url': ding_url})
            else:
                raise NotImplementedError(f'async_task {async_task_name} not exist')
            return View.response.success()
        except (IncomingDataError, NotImplementedError) as e:
            traceback.print_exc()
            return abort(403)
        except RequestFailed as e:
            APP_CELERY.send_task(name='send_dingding_text', args=(DING_TIMOR, traceback.format_exc()))
            traceback.print_exc()

            def parse_error(e: Exception) -> object:
                return json.loads(str(e))

            error = parse_error(e)
            if isinstance(error, dict) and error.get('resp_code') in BoolDataBase.resp_code_biz_data_error:
                return abort(403)
            else:
                return abort(500)
        except Exception as e:
            APP_CELERY.send_task(name='send_dingding_text', args=(DING_TIMOR, traceback.format_exc()))
            traceback.print_exc()
            return abort(500)


app.add_url_rule('/batch-test-huizu/', view_func=BatchTest.as_view('batch_test_huizu'))


@app.route('/login', methods=['post'])  # type: ignore
def login() -> Union[flask.Response, str]:
    return VIEW_INS.login_with_cache()


@app.route('/companies', methods=['get'])
def display_companies() -> flask.Response:
    return View.response.success({ins.uuid: ins.name for ins in Company.get_all()})


@app.route('/products', methods=['get'])
def get_all_products() -> flask.Response:
    return View.response.success({'/'.join((ins.service_name, ins.mode)): ins.name for ins in Product.get_all()})


@app.route('/products/v2', methods=['get'])
def get_all_products_v2() -> flask.Response:
    result = []
    for ins in Product.get_all():
        result.append({
            'index': '/'.join((ins.service_name, ins.mode)),
            'name': ins.name,
            'price': '0',
        })
    return View.response.success(result) # type: ignore


@app.route('/modules', methods=['get'])
def get_all_modules() -> flask.Response:
    VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
    print('get_all_modules >>>>>', FileIni(CODE_BOOK).items(MODULE_INNER_MODULES))
    return VIEW_INS.response.success(dict(FileIni(CODE_BOOK).items(MODULE_INNER_MODULES)))


@app.route('/transfer/official-website', methods=['post']) # type: ignore
def transfer_official_website() -> None:
    msg, phone, name = VIEW_INS.request.get_data('popup', 'iphoneCode', 'name')
    APP_CELERY.send_task('send_dingding_markdown', ('', f'#官网留言+姓名：{name}+手机号：{phone}+留言内容：{msg}'))


# ********************************* view function end ***************************
if __name__ == '__main__':
    CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)
    app.run(port=8889, host='0.0.0.0', debug=False, use_reloader=True, processes=True)


from .base import BangShengBase


class LoanDetail(BangShengBase):
    uri = '/v1/person/loan/request-detail/t2'


# if __name__ == '__main__':
#     conf = {
#         "url": "https://bsapi.bsfit.com.cn",
#         "app_key": "",
#         "app_secret": "",
#     }
#     biz_data = {
#         'idCard': '',
#         'name': '',
#         'mobile': '',
#     }
#     loan = LoanDetail(conf['url'], LoanDetail.uri, conf['app_key'], conf['app_secret'])
#     url = loan.generate_full_url(biz_data)
#     header = loan.generate_header(biz_data)
#     r = loan.request(url, header)
#     print(r.context)



from pysubway.model import db


class Role(db.Model):
    __tablename__ = 'role'

    id = db.Column(db.Integer, primary_key=True)
    uuid = db.Column(db.String(64), nullable=False, unique=True)
    name = db.Column(db.String(32, u'utf8_bin'), nullable=False, unique=True)
    chinese_name = db.Column(db.String(32, u'utf8_bin'), nullable=False, unique=True)
    status = db.Column(db.Integer, nullable=False)
    remark = db.Column(db.String(200, u'utf8_bin'), nullable=False)
    update_at = db.Column(db.DateTime, nullable=False)
    create_at = db.Column(db.DateTime, nullable=False, server_default=db.FetchedValue())



from typing import List

from pysubway.model import db
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import FetchedValue
from sqlalchemy import Integer
from sqlalchemy import String


class RolePermission(db.Model):
    __tablename__ = 'web_permission'
    permission_can = '0'
    status_active = 0

    id = Column(Integer, primary_key=True)
    uuid = Column(String(64), nullable=False, unique=True)
    role_uuid = Column(String(64, u'utf8_bin'), nullable=False)
    block_uuid = Column(String(64, u'utf8_bin'), nullable=False)
    # permission 0 ok, 1 forbidden
    permission = Column(String(64, u'utf8_bin'), nullable=False)
    # status = 0 active, 1 discard
    status = Column(Integer, nullable=False)
    remark = Column(String(200, u'utf8_bin'), nullable=False)
    update_at = Column(DateTime, nullable=False)
    create_at = Column(DateTime, nullable=False, server_default=FetchedValue())

    @classmethod
    def get_all_companies(cls) -> List['RolePermission']:
        return cls.query.all()

    @classmethod
    def the_role_can_do(cls, role_uuid: str, block_uuid: str) -> bool:
        record: RolePermission = cls.query.filter_by(role_uuid=role_uuid, block_uuid=block_uuid).first()
        if not record:
            return False
        return record.permission == cls.permission_can and record.status == cls.status_active



from typing import Any

from .base import AuthenticationBase
from .db import AuthenticationDB
from .file import AuthenticationFile


class AuthenticationFactory:
    """
    file: should configure code book file;
    db:
    Ensure sqlalchemy should be callable when you call the class AuthenticationDB.
    And table block/role/role_permission/user should existed in database.
    """

    def __new__(self, style: str) -> Any:
        if style.lower() == 'file':
            return AuthenticationFile
        elif style.lower() == 'db':
            return AuthenticationDB
        raise NotImplementedError(f'the style {style} is not supported now.')



from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import FetchedValue
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import Text

from pysubway.model import db
from pysubway.utils.env import filter_locals


class User(db.Model):
    __tablename__ = 'user'

    id = Column(Integer, primary_key=True)
    uuid = Column(String(64), nullable=False, unique=True)
    role_uuid = Column(String(64), nullable=False)
    username = Column(String(32, u'utf8_bin'), nullable=False, unique=True)
    password = Column(String(128), nullable=False)
    email = Column(String(128), nullable=False, unique=True)
    phone = Column(String(128), nullable=False, unique=True)
    real_name = Column(String(128), nullable=False)
    company_name = Column(String(128), nullable=False)
    guard_uuid = Column(String(64), nullable=False)
    company_address = Column(String(128), nullable=False)
    business_desc = Column(Text, nullable=False, comment='公司业务描述')
    status = Column(Integer, nullable=False)
    remark = Column(String(200, u'utf8_bin'), nullable=False)
    last_login_time = Column(DateTime, nullable=False)
    update_at = Column(DateTime, nullable=False)
    create_at = Column(DateTime, nullable=False, server_default=FetchedValue())

    @staticmethod
    def generate_hash_password(password: str, salt: str) -> str:
        return ''

    @classmethod
    def get_role(cls, username: str = '', email: str = '', phone: str = '') -> str:
        variables = list(filter_locals(locals(), exclude_keys=('url', 'cls')))
        if len(variables) > 1:
            raise ValueError(f'variables {variables} is unexpected')
        key, value = variables[0]
        print('key, value', key, value)
        result = cls.query.filter(key == value).first()
        return result.role if result else ''



from pysubway.model.user import User

from .base import AuthenticationBase


class AuthenticationDB(AuthenticationBase):

    def __init__(self, salt: str = ''):
        super().__init__()
        self.salt = salt

    def get_password(self, account: str, section: str = None) -> str:
        record: User = User.query.filter_by().first()
        return record.password if record else ''

    def match_password(self, unverified: str, existed: str) -> bool:
        return unverified == User.generate_hash_password(existed, self.salt)



# """
# this is example of using ViewUsedCodeBookAndUrlToken
# """
#
# import json
# import traceback
# from typing import Any, Optional, Dict, Tuple
#
# import flask
# from flask import abort
# from flask.views import MethodView
# from flask_cors import CORS
# from flask_sqlalchemy import SQLAlchemy
# from pysubway.cache import FlaskCache
# from pysubway.errors import IncomingDataError, RequestFailed
# from pysubway.model import call_procedure
# from pysubway.model.guard.company import Company
# from pysubway.model.guard.company_product import CompanyProduct
# from pysubway.model.guard.product import Product
# from pysubway.service.aliyun.dingding.robot import Robot
# from pysubway.service.aliyun.uemail import Email
# from pysubway.service.booldata.base import BoolDataBase
# from pysubway.service.booldata.hz_risk_model import HzRiskModel
# from pysubway.service.booldata.hz_risk_model import batch_test_used_web
# from pysubway.utils.file import File
# from pysubway.utils.file import FileIni
# from pysubway.utils.utime import strftime
# from pysubway.view import View
# from pysubway.view import ViewUsedCodeBookAndUrlToken
#
# BOOLDATA_ACCOUNT = dict(
#     rsa_prv_key="""""",
#     aes_key='',
#     company_uuid='',
# )
#
# PROD_DB = dict(
#     # open
#     host='',
#     user='root',
#     password='',
#     database='',
# )
#
# EMAIL_LIST = [
#     'jack@qq.com',
# ]
#
# EMAIL_PASSWORD = 'i can not tell you'
#
# # remove mypy hints: error: Value of type "object" is not indexable
# DB_CONF: Dict[str, Any] = {
#     'SQLALCHEMY_DATABASE_URI':
#         f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8',
#     'SQLALCHEMY_BINDS': {
#         'guard': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8'
#     },
# }
#
# _DING_TIMOR = 'dingding robot url'
# _DING_CHI_JIAO_DA_XIAN = 'dingding robot url'
#
# app = flask.Flask(__name__)
# CACHE = FlaskCache(app)
# app.config.from_mapping(DB_CONF)
# SQLAlchemy().init_app(app)
# HASH_SALT = 'sdaer3rf'
# KEY_USERNAME = 'userName'
# KEY_PASSWORD = 'password'
# CODE_BOOK = f'{File.this_dir(__file__)}/static/code_book.ini'
# THIS_DIR = File.this_dir(__file__)
# KEY_ACCESS_TOKEN = 'access_token'
# VIEW_INS = ViewUsedCodeBookAndUrlToken(CODE_BOOK,
#                                        CACHE,
#                                        KEY_USERNAME,
#                                        KEY_PASSWORD,
#                                        key_access_token=KEY_ACCESS_TOKEN,
#                                        hash_salt=HASH_SALT)
# # inner platform web section
# MODULE_ACCOUNT_USERNAME = 'account_username'
# MODULE_ACCOUNTS_AUTHENTICATION = 'accounts_authentication'
# # ++++++++++++++++++++++++
# MODULE_EMAIL_NOTIFICATION = 'email_notification'
# OPTION_BATCH_TEST = 'batch_test'
# # ++++++++++++++++++++++++
# MODULE_RECHARGE_MONEY_ACCOUNT = 'recharge_money_account'
# # ++++++++++++++++++++++++
# MODULE_OPEN_TEST_ACCOUNT = 'open_company_test_account'
# # +++++++++++++++++++++++
# MODULE_INNER_MODULES = 'inner_modules'
# MODULE_USER_MANAGER = 'user_manager'
# MODULE_PRODUCT_BATCH_TEST = 'product_batch_test'
#
# TIMOR = Robot(_DING_TIMOR)
# CHI_JIAO_DA_XIAN = Robot(_DING_CHI_JIAO_DA_XIAN)
#
#
# def send_timor(text: str) -> None:
#     try:
#         return TIMOR.send_text(text)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def send_chijiaodaxian(text: str) -> None:
#     try:
#         return CHI_JIAO_DA_XIAN.send_text(text)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def get_real_name(account: str) -> str:
#     result = VIEW_INS.get_code_book_val(MODULE_ACCOUNT_USERNAME, account)
#     if isinstance(result, bool):
#         raise TypeError(f'result {result} should be str')
#     return result
#
#
# def send_email(mail_user: str = EMAIL_LIST[0],
#                mail_pass: str = EMAIL_PASSWORD,
#                title: str = '',
#                content: str = '',
#                specified_receivers: str = '',
#                disable_default_receiver: bool = False,
#                content_is_file: bool = False) -> None:
#     try:
#         if disable_default_receiver:
#             receivers = specified_receivers
#         else:
#             receivers = Email.update_receiver(
#                 VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default'),
#                 specified_receivers
#             )
#         Email(mail_user, mail_pass).send_email(title, content, receivers, content_is_file=content_is_file)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def send_email_attach(mail_user: str = EMAIL_LIST[0],
#                       mail_pass: str = EMAIL_PASSWORD,
#                       title: str = '',
#                       content: str = '',
#                       filename: str = '',
#                       specified_receivers: str = '') -> Any:
#     try:
#         receivers = Email.update_receiver(
#             VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default'),
#             specified_receivers
#         )
#         Email(mail_user, mail_pass).send_email_attach(title, content, filename, receivers)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def open_guard_test_account() -> str:
#     # todo open_guard_test_account
#     return ''
#
#
# @app.before_request
# def verify_token() -> Optional[flask.Response]:
#     """
#     :key cache key is access_token
#     :return:
#     """
#     return VIEW_INS.verify_before_request()
#
#
# class BatchTestHuizu(MethodView):
#
#     @staticmethod
#     def generate_send_msg(real_name, remarks):
#         title = 'Test Application'
#         content = f'applicant: {real_name}\ntest reason：{remarks}' if remarks else ''
#         return title, content
#
#     @classmethod
#     def send_email_batch_test(cls,
#                               account: str = '',
#                               remarks: str = '',
#                               filename: str = '',
#                               specified_receivers: str = '') -> None:
#         real_name = get_real_name(account)
#         title, content = cls.generate_send_msg(real_name, remarks)
#         send_email_attach(title=title,
#                           content=content,
#                           filename=filename,
#                           specified_receivers=specified_receivers)
#
#     @staticmethod
#     def get_company_uuid() -> str:
#         return VIEW_INS.login_cache_password
#
#     @staticmethod
#     def batch_test(account_info: Dict[str, str],
#                    env: str,
#                    company_uuid: str,
#                    biz_data: str,
#                    raise_failed_exception: bool = True,
#                    ) -> Tuple[str, str]:
#         biz_data = biz_data if biz_data.startswith('name') else '\n'.join(('name,phone,idcard', biz_data))
#         return batch_test_used_web(account_info,
#                                    env,
#                                    company_uuid,
#                                    biz_data,
#                                    THIS_DIR,
#                                    raise_failed_exception=raise_failed_exception)
#
#     def post(self) -> str:
#         VIEW_INS.check_operation_authority(MODULE_PRODUCT_BATCH_TEST)
#         account = VIEW_INS.login_cache_username
#         try:
#             company_uuid = self.get_company_uuid()
#             if not CompanyProduct.has_permission_for_product(company_uuid,
#                                                              HzRiskModel.service,
#                                                              HzRiskModel.mode):
#                 raise IncomingDataError(f'company_uuid {company_uuid} do not has_permission_for_product')
#             biz_data, product, remarks, specified_receivers = View.request.get_data('biz_data',
#                                                                                     'product',
#                                                                                     'remarks',
#                                                                                     'email')
#             print('biz_data, product, remarks', biz_data, product, remarks)
#             result, output_file = self.batch_test(BOOLDATA_ACCOUNT,
#                                                   env='prod',
#                                                   company_uuid=company_uuid,
#                                                   biz_data=biz_data)
#             real_name = get_real_name(account)
#             title, content = self.generate_send_msg(real_name, remarks)
#             send_chijiaodaxian(f'{title}\n{content}')
#             self.send_email_batch_test(account=account,
#                                        remarks=remarks,
#                                        filename=output_file)
#             self.send_email_batch_test(account=account,
#                                        remarks='',
#                                        filename=output_file,
#                                        specified_receivers=specified_receivers)
#             return result
#         except IncomingDataError as e:
#             traceback.print_exc()
#             return abort(403)
#         except RequestFailed as e:
#             traceback.print_exc()
#
#             def parse_error(e: Exception) -> object:
#                 return json.loads(str(e))
#
#             error = parse_error(e)
#             if isinstance(error, dict) and error.get('resp_code') in BoolDataBase.resp_code_biz_data_error:
#                 return abort(403)
#             else:
#                 send_timor(traceback.format_exc())
#                 return abort(500)
#         except Exception as e:
#             send_timor(traceback.format_exc())
#             traceback.print_exc()
#             return abort(500)
#
#
# @app.route('/login', methods=['POST'])
# def login() -> flask.Response:
#     return VIEW_INS.login_with_cache()
#
#
# @app.route('/companies', methods=['get'])
# def display_companies() -> flask.Response:
#     return View.response.success({ins.uuid: ins.name for ins in Company.get_all()})
#
#
# @app.route('/products', methods=['get'])
# def get_all_products() -> flask.Response:
#     return View.response.success({'/'.join((ins.service_name, ins.mode)): ins.name for ins in Product.get_all()})
#
#
# @app.route('/products/v2', methods=['get'])
# def get_all_products_v2() -> flask.Response:
#     result = []
#     for ins in Product.get_all():
#         result.append({
#             'index': '/'.join((ins.service_name, ins.mode)),
#             'name': ins.name,
#             'price': '0',
#         })
#     return View.response.success(result)
#
#
# @app.route('/modules', methods=['get'])
# def get_all_modules() -> flask.Response:
#     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
#     print('get_all_modules >>>>>', FileIni(CODE_BOOK).items(MODULE_INNER_MODULES))
#     return VIEW_INS.response.success(dict(FileIni(CODE_BOOK).items(MODULE_INNER_MODULES)))
#
#
# class AddUser(MethodView):
#
#     @staticmethod
#     def send_email(
#             account: str = '',
#             company_name: str = '',
#             recharged_money: int = 0,
#             balanced: int = 0,
#             specified_receivers: str = '') -> None:
#         real_name = get_real_name(account)
#         content = f'{company_name} 于 {strftime()} recharge ￥{recharged_money}，' \
#             f'company current balance {balanced}, operator {real_name}.'
#         send_email(title='Balance Notification', content=content, specified_receivers=specified_receivers)
#
#     def add_module_authority(self, module_name: str, english: str, authority_val: bool = True) -> None:
#         FileIni(CODE_BOOK).set(module_name, english, authority_val)
#
#     def register(self, englishname: str, realname: str) -> None:
#         FileIni(CODE_BOOK).set(MODULE_ACCOUNT_USERNAME, englishname, realname)
#         FileIni(CODE_BOOK).set(MODULE_ACCOUNTS_AUTHENTICATION, englishname, open_guard_test_account())
#
#
# class RechargeCompany(MethodView):
#
#     @staticmethod
#     def send_email(
#             account: str = '',
#             company_name: str = '',
#             recharged_money: int = 0,
#             balanced: int = 0,
#             specified_receivers: str = '') -> None:
#         real_name = get_real_name(account)
#         content = f'{company_name} 于 {strftime()} recharge ￥{recharged_money}，' \
#             f'company current balance {balanced}, operator {real_name}.'
#         send_email(title='Balance Notification', content=content, specified_receivers=specified_receivers)
#
#     def post(self) -> flask.Response:
#         company_uuid, money = View.request.get_data('company', 'money')
#         print(f'account, money >>>>>> {company_uuid},{money}+++++++++++++++++++')
#         VIEW_INS.check_operation_authority(MODULE_RECHARGE_MONEY_ACCOUNT)
#         call_procedure(DB_CONF['SQLALCHEMY_BINDS']['guard'], 'company_recharge_money', company_uuid, money)
#         ins = Company.get_one(uuid=company_uuid)
#         self.send_email(account=VIEW_INS.login_cache_username,
#                         company_name=ins.name,
#                         recharged_money=money,
#                         balanced=ins.balance / 100)
#         return View.response.success({"msg": "success"})
#
#
# class OpenCompanyTestAccount(MethodView):
#
#     @staticmethod
#     def open_company_test_account(email: str) -> None:
#         file = File.join_path(THIS_DIR, 'static', 'open_company_test_account.html')
#         send_email(title='Product Test Env',
#                    content=file,
#                    specified_receivers=email,
#                    content_is_file=True,
#                    disable_default_receiver=True)
#
#     @staticmethod
#     def open_company_prod_account(company_name: str, email: str) -> None:
#         pass
#
#     def post(self) -> flask.Response:
#         company_name, email, comment = View.request.get_data('company', 'email', 'comment')
#         VIEW_INS.check_operation_authority(MODULE_OPEN_TEST_ACCOUNT)
#         self.open_company_test_account(email)
#         operator = get_real_name(VIEW_INS.login_cache_username)
#         send_email(title='Add Account of Product Test Env',
#                    content=f'{company_name} add account of product test env, operator {operator}. comment:{comment}')
#         return View.response.success({})
#
#
# app.add_url_rule('/open/company_test_account', view_func=OpenCompanyTestAccount.as_view('open_company_test_account'))
# app.add_url_rule('/recharge/company', view_func=RechargeCompany.as_view('recharge_company'))
# app.add_url_rule('/batch-test-huizu/', view_func=BatchTestHuizu.as_view('batch_test_huizu'))
#
# if __name__ == '__main__':
#     CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)
#     app.run(port=8889, host='0.0.0.0', debug=False, use_reloader=True, processes=True)



# -*- coding: utf-8 -*-
from typing import Any

from flask_sqlalchemy import SQLAlchemy
from sqlalchemy import create_engine
from sqlalchemy.engine import Engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm import sessionmaker

# Base.metadata.create_all(engine)
SQLALCHEMY_POOL_RECYCLE = 10
db = SQLAlchemy(use_native_unicode='utf8')
Base = declarative_base()

DBSession = sessionmaker()
Scoped = scoped_session(DBSession)


def generate_engine(url: str, pool_recycle: int = SQLALCHEMY_POOL_RECYCLE) -> Engine:
    return create_engine(url, encoding='utf-8', echo=True, pool_recycle=pool_recycle)


def call_procedure(url: str, procedure_name: str, *args: Any) -> None:
    """

    :param url:
    :param procedure_name: just procedure name not call procedure_name
    :param args:
    :return:
    """
    connection = generate_engine(url).raw_connection()
    cursor = connection.cursor()
    try:
        s = cursor.callproc(procedure_name, args=args)
        print(s)
        connection.commit()
    except Exception as e:
        connection.rollback()
    finally:
        cursor.close()
        connection.close()



from pysubway.model import db
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import FetchedValue
from sqlalchemy import Integer
from sqlalchemy import String


class Block(db.Model):
    __tablename__ = 'block'

    id = Column(Integer, primary_key=True)
    uuid = Column(String(64), nullable=False, unique=True)
    name = Column(String(32, u'utf8_bin'), nullable=False)
    chinese_name = Column(String(32, u'utf8_bin'), nullable=False)
    status = Column(Integer, nullable=False)
    remark = Column(String(200, u'utf8_bin'), nullable=False)
    update_at = Column(DateTime, nullable=False)
    create_at = Column(DateTime, nullable=False, server_default=FetchedValue())



from typing import Dict

from pysubway.utils import udict

from .base import SuixingfuBase


class BankcardFourElements(SuixingfuBase):
    verifyType = '04'

    def generate_biz_data(self, name: str, identNo: str, cardNo: str, phone: str) -> Dict[
        str, str]:
        send_data = udict.Dict.filter(locals())
        send_data['verifyType'] = self.verifyType
        return send_data

    def set_biz_data(self, *args: str) -> 'SuixingfuBase':
        self.biz_data = self.generate_biz_data(*args)
        return self



# # example
# import json
# import traceback
# from typing import Any, Optional, Dict, Tuple
#
# import flask
# from celery import Celery
# from flask import abort
# from flask.views import MethodView
# from flask_cors import CORS
# from pysubway.cache import FlaskCache
# from pysubway.errors import IncomingDataError, RequestFailed
# from pysubway.model import call_procedure
# from pysubway.model import db
# from pysubway.model.guard.company import Company
# from pysubway.model.guard.company_product import CompanyProduct
# from pysubway.model.guard.product import Product
# from pysubway.service.aliyun.dingding.robot import Robot
# from pysubway.service.aliyun.uemail import Email
# from pysubway.service.booldata.base import BoolDataBase
# from pysubway.service.booldata.hz_risk_model import HzRiskModel
# from pysubway.service.booldata.hz_risk_model import batch_test_used_web
# from pysubway.utils.file import File
# from pysubway.utils.file import FileIni
# from pysubway.utils.utime import strftime
# from pysubway.view import View
#
# # todo modify the import
# from view import ViewUsedDBAndUrlToken
#
# BOOLDATA_ACCOUNT = dict(
#     rsa_prv_key="""-----BEGIN RSA PRIVATE KEY-----
# MIIEowIBAAKCAQEA3PgggZllwepRByXqAeqrWU4Zx7m+0kvhcbEvpbGTez7PAj8u
# IJRKGnH2QfOkZHsRknamuBtJr9c2Js9yrrWbazNPFQrjekprroBSC7DO+EUHQqxZ
# mV8ldqukT+C/UOHXavNH3FHnrYSgU7yj4I1+AnAzQq33xbSASE9DkTyoEzJnlRjn
# TBopidDVA+P/bxR9z6FBWv8M6P/VAcKamYGIwsx9AbsVhHDd8yOzMDJHOuJza4ku
# KFyjf/N+2Fwc7d6ZbKhRYITio7nerw6/XYrcWhAAyB6XhEp3wfCA5SpgCBWGbdve
# fiPsme5qqirCy2rlPn76Biev3wW2zhVWTw7HDwIDAQABAoIBABA3vdsFMSy8T9og
# dD5TxOO3EblQ7qpsm01G3eJhWBuxjmvxyybk+1NZjeNlSGl/htijELVue0gGmZjb
# nOUpuxBxIZq/w9ZT4/dYv6zP+0DJgDDqiWDyVMOS8WpTanc7PB5DYMDQ2hooI8RB
# kh2HBPqU1Y5NSmQeTVQBTUo5k3RlGbeAE05U3kVfg6j6xf/67rceCOKa6G+Tkp6h
# eebLmuuc19rqg7jBblCsoMgb76P+STlnpvyB8fIYkb1W64bHkX/m4Qj/f4SiDh5t
# 9W3/bmB+2ygsQCaTPGS9y6z8EfnWZT74Fd+o8XuStubEFw0q+b/BdrmEfL5CjEnO
# q57Ep4ECgYEA4/rO82UlKQ6YS0pv4Z7yqPSrJSYwtoazGbGtgp1EHuRk//VEfLOT
# 4Yo665CbBIrTJxfgZlNkSkBoYaZA8KBg1fP56iuCBUvqBRhSvy94E8dNYjLEjX1q
# gskOFrfJ2UaHdp6alctrNSIuWd5q/1LIA4qUBD+mLbnRVWC6dIV2W68CgYEA+CC9
# Zw3M1w+5EYWXgq7D6PiDXMjGgvVn+J0/9nDSxVi41zhnxGQD40h/9eNAai67rJHG
# xvjK313aUVvMJRSXZuR9Y3zwh1Qb6wUdFgDlGUMuJbazVEgNQDrexf9oCGkvlmPH
# /5WDHTOdch07rNaE6/0xnSk45intCIbZy3bEQqECgYACHlWH+3uh6wnNQU7S2OhG
# W6eve7BeMdg+N+F14kI8y0CJBF1zjzOjl+Y+RCS8oRGfPmCOct3utrSBm8rksYjU
# 1CSRYYAeznrJO1Whgy5peKOmcvRSoES6HGYuHd0ZUMd3ebfUBoTjhILLwP5biwhi
# yAniFDlWGXDxiymQZaEY7wKBgQC1r6OAdkZro4OsJNoQQtAJIw9tPUa+VdvP27Ue
# SvbKsnCGhzLURzfhkRDBh6l/FsEtV4Dp6PsEG8Dkmgcdj7pdCSoXU0D8lsXqjVAC
# 8JzQprR/wQwGxsuNII5HBQG0RvEleeLUbzIWi9jZ2jX8FQg3lKpjne+f+UsrVzLb
# HRgQgQKBgDeMsUFxAeNFN9iHu0IqUboe09ULrZ0NZMHOJ3zoFnuus4sEQpNDC32y
# TuzAQumL7z6U5EzzKfdOZX1aAaLoId6+WNjaYkaSD+r/Iz8EzA8V3maip4M2usDl
# NLm+JyWqvj9Ze9y7VOgIqRarqjgNwjBoWysNzKIOMpxxItEWUE6b
# -----END RSA PRIVATE KEY-----""",
#     aes_key='vo8N6ZPawqLDhGj7',
#     company_uuid='01b20490-4a1e-11e9-9eb7-247703d210dc',
# )
#
# DEV_DB = dict(
#     host='47.102.132.18',
#     user='root',
#     password='qwe123456',
#     database='lightning',
# )
#
# PROD_DB = dict(
#     # open
#     host='rm-uf6xfyemz1085v2a48o.mysql.rds.aliyuncs.com',
#     # inner
#     # host='rm-uf6xfyemz1085v2a4.mysql.rds.aliyuncs.com',
#     user='root',
#     password='Qwe123456',
#     database='lighting',
#     inner_platform='inner_platform',
# )
#
# EMAIL_LIST = [
#     'booleandatamsg@shouxin168.com',
#     'booleandatamsg002@shouxin168.com',
#     'booleandatamsg003@shouxin168.com',
#     'booleandatamsg004@shouxin168.com',
#     'booleandatamsg005@shouxin168.com',
# ]
#
# EMAIL_PASSWORD = 'Qwe123456_'
#
# # remove mypy hints: error: Value of type "object" is not indexable
# DB_CONF: Dict[str, Any] = {
#     'SQLALCHEMY_DATABASE_URI':
#         f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["inner_platform"]}?charset=utf8',
#     'SQLALCHEMY_BINDS': {
#         'guard': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8',
#         'default': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}'
#         f'/{PROD_DB["inner_platform"]}?charset=utf8',
#     },
#     'SQLALCHEMY_ECHO': True,
# }
#
# REDIS_URL = 'redis://47.102.132.18:6379/2'
#
# DB_CONF.update(dict(
#     CELERY_TIMEZONE='Asia/Shanghai',
#     CELERY_BROKER_URL=REDIS_URL,
#     CELERY_RESULT_BACKEND=REDIS_URL,
#     CELERY_RESULT_SERIALIZER='json',
#     CELERY_TASK_SERIALIZER='json',
#     CELERY_TASK_RESULT_EXPIRES=3600 * 24,
#     CELERY_ACCEPT_CONTENT=['json', 'json'],
#     # CELERY_INCLUDE=[
#     #     'task_celery.tasks.sw_tasks.company_balance_monitor.main_monitor',
#     #     'task_celery.tasks.sw_tasks.product_called_times_period.main',
#     # ],
# ))
#
# app = flask.Flask(__name__)
# CACHE = FlaskCache(app)
# app.config.from_mapping(DB_CONF)
# db.init_app(app)
#
# celery = Celery(app.name, broker=app.config['CELERY_BROKER_URL'])
# celery.conf.update(app.config)
#
# with app.app_context():
#     db.create_all(app=app)
#     db.session.commit()
# HASH_SALT = 'sdaer3rf'
# KEY_USERNAME = 'userName'
# KEY_PASSWORD = 'password'
# THIS_DIR = File.this_dir(__file__)
# KEY_ACCESS_TOKEN = 'access_token'
# VIEW_INS = ViewUsedDBAndUrlToken(CACHE,
#                                  KEY_USERNAME,
#                                  KEY_PASSWORD,
#                                  key_access_token=KEY_ACCESS_TOKEN,
#                                  hash_salt=HASH_SALT)
# # inner platform web module
# MODULE_ACCOUNT_USERNAME = 'account_username'
# MODULE_ACCOUNTS_AUTHENTICATION = 'accounts_authentication'
# # ++++++++++++++++++++++++
# MODULE_EMAIL_NOTIFICATION = 'email_notification'
# OPTION_BATCH_TEST = 'batch_test'
# # ++++++++++++++++++++++++
# MODULE_RECHARGE_MONEY_ACCOUNT = 'recharge_money_account'
# # ++++++++++++++++++++++++
# MODULE_OPEN_TEST_ACCOUNT = 'open_company_test_account'
# # +++++++++++++++++++++++
# MODULE_INNER_MODULES = 'inner_modules'
# MODULE_USER_MANAGER = 'user_manager'
# MODULE_PRODUCT_BATCH_TEST = 'product_batch_test'
#
# _DING_TIMOR = 'https://oapi.dingtalk.com/robot/send?access_token=ad34f7787f6232a5c82ccdd30210565ae0de828bc8566d0aebf1e5becf03d263'
# _DING_CHI_JIAO_DA_XIAN = 'https://oapi.dingtalk.com/robot/send?access_token=73caa869cc06fc5bb37bf853d7d782784f09dd3705d0c31ade9852f4cf444c25'
# TIMOR = Robot(_DING_TIMOR)
# CHI_JIAO_DA_XIAN = Robot(_DING_CHI_JIAO_DA_XIAN)
#
#
# def send_timor(text: str) -> None:
#     try:
#         return TIMOR.send_text(text)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def send_chijiaodaxian(text: str) -> None:
#     try:
#         return CHI_JIAO_DA_XIAN.send_text(text)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def get_real_name(account: str) -> str:
#     return VIEW_INS.get_code_book_val(MODULE_ACCOUNT_USERNAME, account)
#
#
# def send_email(mail_user: str = EMAIL_LIST[0],
#                mail_pass: str = EMAIL_PASSWORD,
#                title: str = '',
#                content: str = '',
#                specified_receivers: str = '',
#                disable_default_receiver: bool = False,
#                content_is_file: bool = False) -> None:
#     try:
#         if disable_default_receiver:
#             receivers = specified_receivers
#         else:
#             receivers = Email.update_receiver(
#                 VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default'),
#                 specified_receivers
#             )
#         Email(mail_user, mail_pass).send_email(title, content, receivers, content_is_file=content_is_file)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def send_email_attach(mail_user: str = EMAIL_LIST[0],
#                       mail_pass: str = EMAIL_PASSWORD,
#                       title: str = '',
#                       content: str = '',
#                       filename: str = '',
#                       specified_receivers: str = '') -> Any:
#     try:
#         receivers = Email.update_receiver(
#             VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default'),
#             specified_receivers
#         )
#         Email(mail_user, mail_pass).send_email_attach(title, content, filename, receivers)
#     except Exception as e:
#         traceback.print_exc()
#
#
# def open_guard_test_account() -> str:
#     # todo open_guard_test_account
#     return ''
#
#
# @app.before_request
# def verify_token() -> Optional[flask.Response]:
#     """
#     :key cache key is access_token
#     :return:
#     """
#     return VIEW_INS.verify_before_request()
#
#
# class BatchTestHuizu(MethodView):
#
#     @staticmethod
#     def generate_send_msg(real_name, remarks):
#         title = '布尔数据个人信用查询测试申请'
#         content = f'申请者: {real_name}\n测试理由：{remarks}' if remarks else ''
#         return title, content
#
#     @classmethod
#     def send_email_batch_test(cls,
#                               account: str = '',
#                               remarks: str = '',
#                               filename: str = '',
#                               specified_receivers: str = '') -> None:
#         real_name = get_real_name(account)
#         title, content = cls.generate_send_msg(real_name, remarks)
#         send_email_attach(title=title,
#                           content=content,
#                           filename=filename,
#                           specified_receivers=specified_receivers)
#
#     @staticmethod
#     def get_company_uuid() -> str:
#         return VIEW_INS.login_cache_password
#
#     @staticmethod
#     def batch_test(account_info: Dict[str, str],
#                    env: str,
#                    company_uuid: str,
#                    biz_data: str,
#                    raise_failed_exception: bool = True,
#                    ) -> Tuple[str, str]:
#         biz_data = biz_data if biz_data.startswith('name') else '\n'.join(('name,phone,idcard', biz_data))
#         return batch_test_used_web(account_info,
#                                    env,
#                                    company_uuid,
#                                    biz_data,
#                                    THIS_DIR,
#                                    raise_failed_exception=raise_failed_exception)
#
#     def post(self) -> str:
#         VIEW_INS.check_operation_authority(MODULE_PRODUCT_BATCH_TEST)
#         account = VIEW_INS.login_cache_username
#         try:
#             company_uuid = self.get_company_uuid()
#             if not CompanyProduct.has_permission_for_product(company_uuid,
#                                                              HzRiskModel.service,
#                                                              HzRiskModel.mode):
#                 raise IncomingDataError(f'company_uuid {company_uuid} do not has_permission_for_product')
#             biz_data, product, remarks, specified_receivers = View.request.get_data('biz_data',
#                                                                                     'product',
#                                                                                     'remarks',
#                                                                                     'email')
#             print('biz_data, product, remarks', biz_data, product, remarks)
#             result, output_file = self.batch_test(BOOLDATA_ACCOUNT,
#                                                   env='prod',
#                                                   company_uuid=company_uuid,
#                                                   biz_data=biz_data)
#             real_name = get_real_name(account)
#             title, content = self.generate_send_msg(real_name, remarks)
#             send_chijiaodaxian(f'{title}\n{content}')
#             self.send_email_batch_test(account=account,
#                                        remarks=remarks,
#                                        filename=output_file)
#             self.send_email_batch_test(account=account,
#                                        remarks='',
#                                        filename=output_file,
#                                        specified_receivers=specified_receivers)
#             return result
#         except IncomingDataError as e:
#             traceback.print_exc()
#             return abort(403)
#         except RequestFailed as e:
#             traceback.print_exc()
#
#             def parse_error(e: Exception) -> object:
#                 return json.loads(str(e))
#
#             error = parse_error(e)
#             if isinstance(error, dict) and error.get('resp_code') in BoolDataBase.resp_code_biz_data_error:
#                 return abort(403)
#             else:
#                 send_timor(traceback.format_exc())
#                 return abort(500)
#         except Exception as e:
#             send_timor(traceback.format_exc())
#             traceback.print_exc()
#             return abort(500)
#
#
# @app.route('/login', methods=['POST'])
# def login() -> flask.Response:
#     return VIEW_INS.login_with_cache()
#
#
# @app.route('/companies', methods=['get'])
# def display_companies() -> flask.Response:
#     return View.response.success({ins.uuid: ins.name for ins in Company.get_all()})
#
#
# @app.route('/products', methods=['get'])
# def get_all_products() -> flask.Response:
#     return View.response.success({'/'.join((ins.service_name, ins.mode)): ins.name for ins in Product.get_all()})
#
#
# @app.route('/products/v2', methods=['get'])
# def get_all_products_v2() -> flask.Response:
#     result = []
#     for ins in Product.get_all():
#         result.append({
#             'index': '/'.join((ins.service_name, ins.mode)),
#             'name': ins.name,
#             'price': '0',
#         })
#     return View.response.success(result)
#
#
# @app.route('/modules', methods=['get'])
# def get_all_modules() -> flask.Response:
#     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
#     return VIEW_INS.response.success(dict(FileIni(CODE_BOOK).items(MODULE_INNER_MODULES)))
#
#
# # @app.route('add/user', methods=['post'])
# # def add_user():
# #     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
#
#
# # class AddUser(MethodView):
# #
# #     @staticmethod
# #     def send_email(
# #             account: str = '',
# #             company_name: str = '',
# #             recharged_money: int = 0,
# #             balanced: int = 0,
# #             specified_receivers: str = '') -> None:
# #         real_name = get_real_name(account)
# #         content = f'{company_name} 于 {strftime()} 充值 {recharged_money} 元，' \
# #             f'该公司当前账户余额为 {balanced}, 充值操作者为 {real_name}.'
# #         send_email(title='公司充值邮件提醒', content=content, specified_receivers=specified_receivers)
# #
# #     def add_module_authority(self, module_name: str, english: str, authority_val: bool = True) -> None:
# #         FileIni(CODE_BOOK).set(module_name, english, authority_val)
# #
# #     def register(self, englishname: str, realname: str) -> None:
# #         FileIni(CODE_BOOK).set(MODULE_ACCOUNT_USERNAME, englishname, realname)
# #         FileIni(CODE_BOOK).set(MODULE_ACCOUNTS_AUTHENTICATION, englishname, open_guard_test_account())
# #
# #     # def post(self) -> flask.Response:
# #     #     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
# #     #     englishname, realname, email, comment, moduleArr = View.request.get_data('englishname',
# #     #                                                                              'realname',
# #     #                                                                              'email',
# #     #                                                                              'comment',
# #     #                                                                              'moduleArr')
# #     #     self.register(englishname, realname)
# #     #     self.send_email(
# #     #         account=VIEW_INS.login_cache_username,
# #     #         company_name=ins.name,
# #     #         recharged_money=money,
# #     #         balanced=ins.balance / 100)
# #     #     return View.response.success({"msg": "操作成功"})
#
#
# class RechargeCompany(MethodView):
#
#     @staticmethod
#     def send_email(
#             account: str = '',
#             company_name: str = '',
#             recharged_money: int = 0,
#             balanced: int = 0,
#             specified_receivers: str = '') -> None:
#         real_name = get_real_name(account)
#         content = f'{company_name} 于 {strftime()} 充值 {recharged_money} 元，' \
#             f'该公司当前账户余额为 {balanced}, 充值操作者为 {real_name}.'
#         send_email(title='公司充值邮件提醒', content=content, specified_receivers=specified_receivers)
#
#     def post(self) -> flask.Response:
#         company_uuid, money = View.request.get_data('company', 'money')
#         print(f'account, money >>>>>> {company_uuid},{money}+++++++++++++++++++')
#         VIEW_INS.check_operation_authority(MODULE_RECHARGE_MONEY_ACCOUNT)
#         call_procedure(DB_CONF['SQLALCHEMY_BINDS']['guard'], 'company_recharge_money', company_uuid, money)
#         ins = Company.get_one(uuid=company_uuid)
#         self.send_email(account=VIEW_INS.login_cache_username,
#                         company_name=ins.name,
#                         recharged_money=money,
#                         balanced=ins.balance / 100)
#         return View.response.success({"msg": "操作成功"})
#
#
# class OpenCompanyTestAccount(MethodView):
#
#     @staticmethod
#     def open_company_test_account(email: str) -> None:
#         file = File.join_path(THIS_DIR, 'static', 'open_company_test_account.html')
#         send_email(title='布尔数据测试环境信息',
#                    content=file,
#                    specified_receivers=email,
#                    content_is_file=True,
#                    disable_default_receiver=True)
#
#     @staticmethod
#     def open_company_prod_account(company_name: str, email: str) -> None:
#         pass
#
#     def post(self) -> flask.Response:
#         company_name, email, comment = View.request.get_data('company', 'email', 'comment')
#         VIEW_INS.check_operation_authority(MODULE_OPEN_TEST_ACCOUNT)
#         self.open_company_test_account(email)
#         operator = get_real_name(VIEW_INS.login_cache_username)
#         send_email(title='测试环境开通账户', content=f'{company_name} 开通布尔数据测试环境, 操作者 {operator}. 备注信息:{comment}')
#         return View.response.success({})
#
#
# app.add_url_rule('/open/company_test_account', view_func=OpenCompanyTestAccount.as_view('open_company_test_account'))
# app.add_url_rule('/recharge/company', view_func=RechargeCompany.as_view('recharge_company'))
# app.add_url_rule('/batch-test-huizu/', view_func=BatchTestHuizu.as_view('batch_test_huizu'))
#
# if __name__ == '__main__':
#     CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)
#     app.run(port=8889, host='0.0.0.0', debug=False, use_reloader=True, processes=True)
